import tkinter as tk
import tkinter.filedialog as fd
import pandas as pd
import numpy as np
import pystan
import matplotlib.pyplot as plt
import seaborn as sb

def askinfilename(msg = None, types = [('', '*.*')]):
    """
            入力用ファイル名の設定
    """
    rt = tk.Tk()
    rt.withdraw()
    flnm = fd.askopenfilename(title = msg, filetypes = types)
    rt.destroy()
    return flnm

def askoutfilename(msg = None, types = [('', '*.*')], deftype = '.txt'):
    """
            出力用ファイル名の設定
    """
    rt = tk.Tk()
    rt.withdraw()
    flnm = fd.asksaveasfilename(title = msg, filetypes = types,
                                           defaultextension = deftype)
    rt.destroy()
    return flnm


finnm = askinfilename('入力データExcelファイル', types = [('Excel(*.xlsx)', '*.xlsx')])
foutnm = askoutfilename(msg = '画像保存ファイル', types = [('画像 (*.png)', '*.png')],
                       deftype = '.png')

xlsx_fl = pd.ExcelFile(finnm)
data_xlsx = pd.read_excel(xlsx_fl,  header = None)
data = data_xlsx.values

VarName = data[0,1]
X = data[1:, 1]
X = list(X)
print(X)

DataStan = {'N': len(X), 'X': X}

sm = pystan.StanModel(file = 'OneVarNormal.stan')
fit = sm.sampling(data = DataStan, n_jobs = 1)
print(fit)

mu_025, mu_50, mu_975 = np.percentile(fit['mu'], [2.5, 50, 97.5])

#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

plt.title('平均値の事後分布' + \
          '\nMed = {0:.2f},  95%CI = [{1:.2f}, {2:.2f}]'.format(
              mu_50, mu_025, mu_975), fontsize = 20)
sb.kdeplot(fit['mu'])
plt.xlabel(VarName + 'の平均値', fontsize = 16)
plt.yticks([])
plt.tight_layout()
plt.savefig(foutnm[:-4] + '.png')
plt.show()

sgm_025, sgm_50, sgm_975 = np.percentile(fit['sgm'], [2.5, 50, 97.5])
plt.title('標準偏差の事後分布' + \
          '\nMed = {0:.2f},   95%CI = [{1:.2f}, {2:.2f}]'.format(
              sgm_50, sgm_025, sgm_975), fontsize = 20)
sb.kdeplot(fit['sgm'])
plt.xlabel(VarName + 'の標準偏差', fontsize = 14)
plt.yticks([])
plt.tight_layout()
plt.savefig(foutnm[:-4] + '_sgm.png')
plt.show()
